package org.beangle.security.monitor;

import java.util.Collection;
import java.util.List;

import org.beangle.security.User;
import org.beangle.security.UserCategory;

public interface SessionController {

	/**
	 * 检查能否注册
	 * 
	 * @param auth
	 * @return
	 */
	public boolean checkAuthenticationAllowed(Authentication auth);

	/**
	 * 注册
	 * 
	 * @param authentication
	 */
	public void registerAuthentication(Authentication authentication);

	/**
	 * 删除指定会话id对应的用户活动
	 * 
	 * @param sessionId
	 */
	public OnlineActivity removeAuthentication(String sessionId);

	/**
	 * 查询指定会话id的在线活动
	 * 
	 * @param sessionId
	 * @return
	 */
	public OnlineActivity getOnlineActivity(String sessionId);

	/**
	 * 是否已经注册
	 * 
	 * @param principal
	 * @return
	 */
	public boolean isRegisted(Object principal);

	/**
	 * 查询所有在线活动
	 * 
	 * @return
	 */
	public List<OnlineActivity> getOnlineActivities();

	/**
	 * 查询允许登陆用户的最大值
	 */
	public int getMax();

	/**
	 * 返回某一类用户的最大用户数
	 * 
	 * @param category
	 * @return
	 */
	public int getMax(UserCategory category);

	/**
	 * 返回某类用户的不活动时间间隔(min)
	 * 
	 * @param category
	 * @return
	 */
	public int getInactiveInterval(UserCategory category);

	/**
	 * 是否到达最大用户
	 * 
	 * @return
	 */
	public boolean isMaxArrived();

	/**
	 * 是否某类用户到达最大数
	 * 
	 * @return
	 */
	public boolean isMaxArrived(UserCategory category);

	/**
	 * 在线用户数
	 */
	public int getOnlineCount();

	/**
	 * 查询某类用户的在线人数
	 * 
	 * @param category
	 * @return
	 */
	public int getOnlineCount(UserCategory category);

	/**
	 * 某用户的在线信息
	 * 
	 * @param category
	 * @return
	 */
	public Collection<OnlineActivity> getOnlineActivities(User user);

	/**
	 * 切换身份
	 * 
	 * @param userId
	 * @param category
	 *            切换身份
	 * @return
	 */
	public void changeCategory(String sessionId, UserCategory category);

}
