package org.beangle.security.monitor.auth.provider;

import org.beangle.model.persist.EntityDao;
import org.beangle.security.User;
import org.beangle.security.UserCategory;
import org.beangle.security.monitor.Authentication;
import org.beangle.security.monitor.AuthenticationException;
import org.beangle.security.monitor.auth.ui.UserDetails;
import org.beangle.security.service.UserService;

public abstract class AbstractAuthenticationProvider implements AuthenticationProvider {

	protected UserService userService;

	protected EntityDao entityDao;
	
	public void setUserService(UserService userService) {
		this.userService = userService;
	}

	public User attachToUser(Authentication auth) {
		User user = userService.get((String) auth.getPrincipal());
		if (null == user) {
			throw new AuthenticationException(Authentication.ERROR_NOTEXIST);
		}
		if (!user.isEnabled()) {
			throw new AuthenticationException(Authentication.ERROR_UNACTIVE);
		}
		if (null == auth.getDetails()) {
			auth.setDetails(new UserDetails());
		}
		UserDetails details = (UserDetails) auth.getDetails();
		details.setUserid(user.getId());
		details.setFullname(user.getFullname());
		details.setCategory(entityDao.get(UserCategory.class, user.getDefaultCategory().getId()));
		return user;
	}

	public void setEntityDao(EntityDao entityDao) {
		this.entityDao = entityDao;
	}

}
