package org.beangle.security.monitor.auth.provider;

import java.util.Iterator;
import java.util.List;

import org.beangle.commons.collection.ListUtil;
import org.beangle.security.monitor.Authentication;
import org.beangle.security.monitor.AuthenticationBreakException;
import org.beangle.security.monitor.AuthenticationException;
import org.beangle.security.monitor.AuthenticationManager;
import org.beangle.security.monitor.OnlineActivity;
import org.beangle.security.monitor.SessionController;
import org.beangle.security.monitor.auth.AuthenticationListener;
import org.beangle.security.monitor.auth.session.ConcurrentLoginException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;

public class ProviderManager implements AuthenticationManager, InitializingBean {

	protected final Logger logger = LoggerFactory.getLogger(ProviderManager.class);

	protected List<AuthenticationProvider> providers = ListUtil.newArrayList();

	protected List<AuthenticationListener> listeners = ListUtil.newArrayList();

	protected SessionController sessionController;

	public void afterPropertiesSet() throws Exception {
		if (providers.isEmpty()) {
			throw new RuntimeException("authentication provider list is empty");
		}
		logger.info("providers:" + providers);
	}

	public Authentication authenticate(Authentication auth) throws AuthenticationException {
		Iterator<AuthenticationProvider> iter = getProviders().iterator();
		Class<?> toTest = auth.getClass();
		AuthenticationException lastException = null;

		while (iter.hasNext()) {
			AuthenticationProvider provider = iter.next();
			if (!provider.supports(toTest)) {
				continue;
			}
			Authentication result;
			try {
				result = provider.authenticate(auth);
			} catch (AuthenticationException ae) {
				lastException = ae;
				if (ae instanceof AuthenticationBreakException) {
					break;
				} else {
					continue;
				}
			}

			if (sessionController.checkAuthenticationAllowed(result)) {
				sessionController.registerAuthentication(result);
				for (AuthenticationListener listener : listeners) {
					listener.afterAuthenticate(auth);
				}
				return result;
			} else {
				lastException = new ConcurrentLoginException(Authentication.ERROR_OVERMAX);
				break;
			}
		}

		if (lastException == null) {
			lastException = new ProviderNotFoundException();
		}
		throw lastException;
	}

	public OnlineActivity remove(String sessionId) {
		OnlineActivity activity = sessionController.removeAuthentication(sessionId);
		if (null != activity) {
			for (AuthenticationListener listener : listeners) {
				listener.afterRemove(activity);
			}
		}
		return activity;
	}

	public List<AuthenticationProvider> getProviders() {
		return providers;
	}

	public void setProviders(List<AuthenticationProvider> providers) {
		this.providers = providers;
	}

	public void setSessionController(SessionController sessionController) {
		this.sessionController = sessionController;
	}

	public SessionController getSessionController() {
		return sessionController;
	}

	public List<AuthenticationListener> getListeners() {
		return listeners;
	}

	public void setListeners(List<AuthenticationListener> listeners) {
		this.listeners = listeners;
	}

}
