package org.beangle.security.monitor.auth.provider.ldap;

import org.beangle.security.monitor.Authentication;
import org.beangle.security.monitor.AuthenticationException;
import org.beangle.security.monitor.auth.provider.AbstractAuthenticationProvider;
import org.beangle.security.monitor.auth.provider.UserNamePasswordAuthentication;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractLdapAuthenticationProvider extends AbstractAuthenticationProvider {
	private static final  Logger logger = LoggerFactory.getLogger(AbstractLdapAuthenticationProvider.class);

	final public Authentication authenticate(Authentication auth) throws AuthenticationException {
		logger.debug("Authentication using {}", getClass());
		if (doVerify(auth)) {
			attachToUser(auth);
			return auth;
		} else {
			throw new BadLdapCredentialsException();
		}
	}

	abstract protected boolean doVerify(Authentication auth);

	public boolean supports(Class<?> authTokenType) {
		return (UserNamePasswordAuthentication.class.isAssignableFrom(authTokenType));
	}

	public String toString() {
		return getClass().getName();
	}
}
