package org.beangle.security.monitor.auth.provider.sso;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.beangle.security.monitor.Authentication;
import org.beangle.security.monitor.AuthenticationException;
import org.beangle.security.monitor.auth.provider.AbstractAuthenticationProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractSsoAuthenticationProvider extends AbstractAuthenticationProvider {
	private static final Logger logger = LoggerFactory
			.getLogger(AbstractSsoAuthenticationProvider.class);

	public Authentication authenticate(Authentication auth) throws AuthenticationException {
		logger.debug("Authentication using {}", getClass());
		SsoAuthentication ssoau = (SsoAuthentication) auth;
		String userName = doGetUserLoginName(ssoau.getRequest());
		if (StringUtils.isEmpty(userName)) {
			throw new AuthenticationException("sso user not found!");
		} else {
			auth.setPrincipal(userName);
			attachToUser(auth);
			return auth;
		}
	}

	abstract protected String doGetUserLoginName(HttpServletRequest request);

	public boolean supports(Class<?> authTokenType) {
		return (SsoAuthentication.class.isAssignableFrom(authTokenType));
	}

	public String toString() {
		return getClass().getName();
	}
}
