/*
 *
 * Copyright c 2005-2009
 * Licensed under the Apache License, Version 2.0 (the "License")
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 */
/********************************************************************************
 * @author chaostone
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * chaostone             2006-8-28            Created
 *  
 ********************************************************************************/
package org.beangle.security.monitor.impl;

import javax.servlet.http.HttpSession;

import org.beangle.security.UserCategory;
import org.beangle.security.monitor.Authentication;
import org.beangle.security.monitor.AuthenticationException;
import org.beangle.security.monitor.AuthenticationManager;
import org.beangle.security.monitor.AuthorityManager;
import org.beangle.security.monitor.SecurityManager;
import org.beangle.security.monitor.auth.provider.rememberme.RememberMeService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 缺省的监听实现
 * 
 * @author chaostone
 * 
 */
public class DefaultSecurityManager implements SecurityManager {

	protected static final Logger logger = LoggerFactory.getLogger(DefaultSecurityManager.class);

	/** 允许从cookie中进行登录 */
	protected boolean enableRememberMe = true;

	protected AuthorityManager authorityManager;

	protected AuthenticationManager authenticationManager;

	protected RememberMeService rememberMeService;

	public Authentication authenticate(Authentication auth) throws AuthenticationException {
		return authenticationManager.authenticate(auth);
	}

	/**
	 * 注销用户<br>
	 * 1)根据用户session删除用户登陆记录<br>
	 * 2)删除权限<br>
	 * 3)clear session attributes<br>
	 * 4)持久化会话记录 but not invalidate session
	 * 
	 * @param sessionId
	 */
	public void logout(HttpSession session) {
		HttpSessionHolder.setSession(session);
		authenticationManager.remove(session.getId());
	}

	public void changeCategory(String sessionId, UserCategory category) {
		authenticationManager.getSessionController().changeCategory(sessionId, category);
	}

	public boolean isPublicResource(String actionName) {
		return authorityManager.isPublic(actionName);
	}

	/**
	 * 资源是否被授权<br>
	 */
	public boolean isAuthorized(Long userId, String actionName) {
		return authorityManager.isAuthorized(userId, actionName);
	}

	public RememberMeService getRememberMeService() {
		return rememberMeService;
	}

	public AuthenticationManager getAuthenticationManager() {
		return authenticationManager;
	}
	
	public AuthorityManager getAuthorityManager() {
		return authorityManager;
	}

	public void setRememberMeService(RememberMeService rememberMeService) {
		this.rememberMeService = rememberMeService;
	}

	public boolean isEnableRememberMe() {
		return enableRememberMe;
	}

	public void setEnableRememberMe(boolean enableRememberMe) {
		this.enableRememberMe = enableRememberMe;
	}

	public void setAuthorityManager(AuthorityManager authorityManager) {
		this.authorityManager = authorityManager;
	}

	public void setAuthenticationManager(AuthenticationManager authenticationManager) {
		this.authenticationManager = authenticationManager;
	}

}
